/* Copyright (C) 2003 Albert Seward
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sf.jlink;

import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;


public class JLink {
        public static final  String VERSION = "0.2";
        public static final String NAME = "Hell Link";
        public static final String ABOUT = 
        JLink.NAME + " " + JLink.VERSION + " for KiSS DP-xxxx\n This is a modified version of Jlink v0.5\n" + 
        "Copyright (C) 2003 Albert Seward\n\n" +
        "This program is free software; you can redistribute\n"+
        "it and/or modify it under the terms of the GNU\n" +
        "General Public License as published by the Free\n" + 
        "Software Foundation; either version 2 of the\n" + 
        "License, or (at your option) any later version.\n" +
        "See the GNU General Public License for more details.\n\n" +
        "Please report bugs and suggestions to\n" +
        "Albert Seward (alse7905@student.uu.se)\n" +
        "Se JLink Homepage for Updates\n" +
        "http://user.it.uu.se/~alse7905/JLink/" ;
        public static final String TITLE = NAME + " - KiSS DP-XXXX";
        public static final String JVM_REQUIRED_VERSION = "1.4.2";
                
	private static String configFile = "HLink.conf";
        public static boolean enableSysTray = false;
        private static boolean verbose = false;
        private static boolean gui = true;
        public static boolean lafmod = true;
        
              
        public  JLink(String[] args)
        {
          main(args);
        }
	public static void main(String[] args) {
                int i = 0;
                JWindow mainWindow = null;
                JConfig config = null;
                ServerSocket serverSocket = null;
                String lookAndFeel = UIManager.getSystemLookAndFeelClassName();     
                String arg;
                boolean iconified = false;
                
                // Parsing Command Line Arguments //
                while(i < args.length && args[i].startsWith("-")) {
                    arg = args[i++];
                    if (arg.equals("-help")) {
                        printHelp();
                        System.exit(0);
                    } else if(arg.equals("-systray")) {
                        enableSysTray = true;
                    } else if(arg.equals("-nolaf")) {
                        lookAndFeel = null;
                        lafmod = false;
                    } else if(arg.equals("-laf")) {
                        if (i < args.length) {
                            lookAndFeel = args[i++];
                            lafmod = false;
                        } else {
                            System.err.println("-laf requires a look and feel");
                            System.exit(1);
                        }
                    } else if(arg.equals("-config")) {
                        if (i < args.length) {
                            configFile = args[i++];
                        } else {
                            System.err.println("-config requires a filename");
                            System.exit(1);
                        }
                    } else {
                        for(int j = 1; j < arg.length(); j++) {
                            char flag = arg.charAt(j);
                            switch (flag) {
                                case 'v': verbose = true; break;
                                case 'i': iconified = true; break;
                                case 'g': gui = false; break;
                                case 'h': printHelp(); System.exit(0);
                                default:
                                 System.err.println(flag + " illegal option\ntry -help");
                                 System.exit(1);
                                 break;
                            }
                        }
                    }
                }
                if (i != args.length || (!enableSysTray && iconified)) {
                    printUsage();
                    System.exit(1);
                }
                
                debug(System.getProperties().toString());
                
                jVMCheck();
                                
                if(!gui)
                    System.out.println(NAME + " " + VERSION);               
                else if(lookAndFeel != null) {
                    try {
                        UIManager.setLookAndFeel(lookAndFeel);
                    } catch (Exception e) {
                        error("Unable to load LookAndFeel " + lookAndFeel, null, true);
                        e.printStackTrace();
                    }
                }  
                    
                config = new JConfig(configFile);
                try {
                    config.load();
                } catch(Exception e) {
                    error("Error loading configfile " + configFile, null, false);
                    try {
                        config.save();
                    } catch(Exception err) {
                        error("Unabale to create configfile " + configFile, null, false);
                    }   
                }
                             
                try {                      
                    if(gui) {
                        mainWindow = new JWindow(config);
                        /* if(!verbose) {
                            System.out.close();
                            System.in.close();
                            System.err.close();
                        } */
                    }    
                    ClientDetector detector = new ClientDetector(config);
                    ClientListener server = new ClientListener(config, mainWindow);
                    if(gui && !iconified)
                        mainWindow.setVisible(true);
                    server.start();
                    detector.start();
                } catch(Exception e) {
                    error(e);
                } 
                
        }
             
        public static void debug(String msg) {
            if(verbose)
                System.err.println(msg);
        }

        public static void error(Exception e) {
             StackTraceElement l[] = e.getStackTrace();
             String err = e.toString() + "\n";
             for(int i = 0; i < l.length; i++) {
                err += "   " + l[i].toString() + "\n";
             }
             error("Unexpected exception \n" + err + "\nPlease report this error if you think it is a bug\nto Albert Seward alse7905@student.uu.se\nThanks", null, true);
        } 
              
        public static void error(String msg, JFrame frame, boolean fatal) {
            if(gui)
                JOptionPane.showMessageDialog(frame, msg,(fatal?"Fatal error":"Error"), (fatal?JOptionPane.ERROR_MESSAGE:JOptionPane.WARNING_MESSAGE));
            else
                System.err.println((fatal?"Fatal error: ":"Error: ") + msg);
            if(fatal)
                System.exit(1);
        }
        
        private static void jVMCheck() {
            try {
                String jVMVersion[] = System.getProperty("java.version").split("\\.|_");
                String requiredVersion[] = JVM_REQUIRED_VERSION.split("\\.");
                for(int i = 0; i < 3; i++) {
                    if(Integer.parseInt(jVMVersion[i]) < Integer.parseInt(requiredVersion[i])) {
                        error("Your Java Runtime Environment version is " + System.getProperty("java.version") + "\n It is recomended that you use at least version " + JVM_REQUIRED_VERSION, null, false);
                        break;
                    }
                }
            } catch(Exception err) {
                error("Unable to determine your Java version", null, false); 
            }   
        }
        
        private static void printUsage() {
            System.out.println("Usage: " + NAME + " [-help] [-v] [-g] [-systray [-i]] [-nolaf | -laf look] [-config filename]");  
        }
        
        private static void printHelp() {
            printUsage();
            System.out.println("");
            System.out.println("  -v         Verbose output");
            System.out.println("  -g         Disable gui");
            System.out.println("  -i         Start iconified to systray");
            System.out.println("  -systray   Make use of systray");
            System.out.println("  -nolaf     Disable look and feel. Use java default");
            System.out.println("  -laf x     Use specified look and feel");
            System.out.println("             ex. -laf com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
            System.out.println("                 -laf com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            System.out.println("                 -laf javax.swing.plaf.metal.MetalLookAndFeel");   
            System.out.println("                 -laf com.sun.java.swing.plaf.motif.MotifLookAndFeel");  
            System.out.println("  -config x  Load options from alternative configurationfile");
        }
        
  

}

