/* Copyright (C) 2003 Albert Seward
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sf.jlink;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;


public class OptionsDialog extends JDialog implements WindowListener, ActionListener {
    Mount mount;
    private JTextField menuTextField;
    private JTextField localTextField;
    private JCheckBox enablePLSCheckBox;
    private JCheckBox enableM3UCheckBox; 

    public OptionsDialog(Frame parent, Mount mount) {
        super(parent, "Edit Share", true);
        this.mount = mount != null?mount:new Mount();
        initComponents();
    }
    
    public Mount getMount() {
        return mount;
    }
    
    private void initComponents() {
        setModal(true);
        getContentPane().setLayout(new GridBagLayout());             
        JPanel pathPanel = new JPanel();
        pathPanel.setLayout(new GridBagLayout());
        pathPanel.setOpaque(false);
        pathPanel.setBorder(new TitledBorder("Share"));   
        pathPanel.add(new JLabel("Mount path"), new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
        menuTextField = new JTextField(mount.getMountPoint());
        pathPanel.add(menuTextField, new GridBagConstraints(1, 0, 2, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 0, 0));
        pathPanel.add(new JLabel("Local path"), new GridBagConstraints(0,1, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
        localTextField = new JTextField(mount.getMapPath());
        pathPanel.add(localTextField, new GridBagConstraints(1, 1, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 0, 0));
        JButton browseButton = new JButton("Browse");
        if(JLink.lafmod)
            browseButton.setPreferredSize(new Dimension(78, 22));
        browseButton.setActionCommand("browse button");
        browseButton.addActionListener(this);
        pathPanel.add(browseButton, new GridBagConstraints(2, 1, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 10, 5, 10), 0, 0));
        getContentPane().add(pathPanel, new GridBagConstraints(0, 0, 2, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 5, 10), 0, 0));    
        JPanel directoryPanel = new JPanel();
        directoryPanel.setOpaque(false);
        directoryPanel.setLayout(new GridLayout(2, 1));
        directoryPanel.setBorder(new TitledBorder("Directory Options"));
        enablePLSCheckBox = new JCheckBox("Show .pls playlist files as directories", mount.supportPLS);
        enablePLSCheckBox.setOpaque(false);
        directoryPanel.add(enablePLSCheckBox);
        enableM3UCheckBox = new JCheckBox("Show .m3u playlist files as directories", mount.supportM3U);
        enableM3UCheckBox.setOpaque(false);
        directoryPanel.add(enableM3UCheckBox);
        getContentPane().add(directoryPanel, new GridBagConstraints(0, 1, 2, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 10, 5, 10), 0, 0)); 
        
        JButton okButton = new JButton("    OK    ");
        okButton.setActionCommand("ok button");
        okButton.addActionListener(this);
        getContentPane().add(okButton, new GridBagConstraints(0, 2, 1, 1, 1, 0, GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));  
        
        JButton cancelButton = new JButton(" Cancel ");
        cancelButton.setActionCommand("cancel button");
        cancelButton.addActionListener(this);
        getContentPane().add(cancelButton, new GridBagConstraints(1, 2, 1, 1, 0, 0, GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE, new Insets(5, 5, 5, 10), 0, 0));  
        
        
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(this);
        pack();
        setSize(400, (int)getPreferredSize().getHeight());
        
        setResizable(false);
        Dimension sc = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation((sc.width - getWidth()) / 2, (sc.height - getHeight()) / 2);
    }
    
    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand().equals( "ok button" )) {
            String m = menuTextField.getText();
             if(!FileSystem.isValidStart(m)) {
                JLink.error("Your path has to start with " + 
                FileSystem.pathSeparatorChar + "AUDIO or " + 
                FileSystem.pathSeparatorChar + "VIDEO or " + 
                FileSystem.pathSeparatorChar + "PICTURE", null, false);
                return;
            }
            try {
                mount.setMapPath(localTextField.getText());                 
            } catch (DirectoryNotFoundException err) {
                JLink.error("Unable to find path \n\"" + localTextField.getText() + "\"", null, false);
                return;
            }            
            mount.setMountPoint(menuTextField.getText());          
            mount.supportPLS = enablePLSCheckBox.isSelected();
            mount.supportM3U = enableM3UCheckBox.isSelected(); 
            setVisible(false);
            dispose();
        } else if(e.getActionCommand().equals( "cancel button" )) {
            mount = null;
            setVisible(false);
            dispose();
        } else if(e.getActionCommand().equals( "browse button" )) {
            JFileChooser fileChooser = new JFileChooser(mount.getMapPath());
            fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            switch(fileChooser.showOpenDialog(this)) {
                case JFileChooser.ERROR_OPTION:
                    JLink.error("This is a bug. Please Report (JFileChooser.ERROR_OPTION)", null, false);
                case JFileChooser.CANCEL_OPTION: return;
                case JFileChooser.APPROVE_OPTION:
                    String path = fileChooser.getSelectedFile().getAbsolutePath();
                    localTextField.setText(path);
                break;
            }
        }
    }
    
    public void windowClosing(WindowEvent e) {
        mount = null;
        setVisible(false);
        dispose();
    }
      
    public void windowActivated(WindowEvent e) {
    }
    
    public void windowClosed(WindowEvent e) {
    }
    
    public void windowDeactivated(WindowEvent e) {
    }
    
    public void windowDeiconified(WindowEvent e) {
    }
    
    public void windowIconified(WindowEvent e) {
    }
    
    public void windowOpened(WindowEvent e) {
    }  
}
