/***************************************************************************
                                menubase.cpp
                             ------------------
    begin                : 04.03.2003
    copyright            : (C) 2003 by Tamas Bara
    email                : tbara@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "menubase.h"
#include "transceiver.h"
#include "systraymanager.h"

MenuBase::MenuBase( int id, Transceiver* pTransceiver, SysTrayManager* pManager )
{
    m_id               = id;
    m_pTransceiver     = pTransceiver;
    m_pManager         = pManager;

    m_pMenu = new QPopupMenu();
    connect( m_pMenu, SIGNAL( activated( int ) ), SLOT( itemSelected( int ) ) );
}

MenuBase::~MenuBase()
{
    delete m_pMenu;
}

void MenuBase::itemSelected( int id )
{
    QValueList< int >::iterator iter = m_checkableItems.begin();
    while( iter != m_checkableItems.end() )
    {
        if( *iter == id ) break;
        iter++;
    }

    if( iter != m_checkableItems.end() )
    {
        m_pMenu->setItemChecked( id, !m_pMenu->isItemChecked( id ) );
    }

    m_pTransceiver->sendItemSelected( m_id, id );
}

void MenuBase::enableItem( int id, bool enable )
{
    m_pMenu->setItemEnabled( id, enable );
}

void MenuBase::checkItem( int id, bool check )
{
    m_pMenu->setItemChecked( id, check );
}

void MenuBase::setItemLabel( int id, QString& label )
{
    m_pMenu->changeItem( id, label );
}

void MenuBase::removeItem( int id )
{
    m_pMenu->removeItem( id );

    int newId = id;
    int pos = m_pMenu->count() - id - 1;
    for( int i = pos; i > -1; i-- ) m_pMenu->setId( i, newId++ );
}

void MenuBase::removeAll()
{
    m_pMenu->clear();
    m_checkableItems.clear();
}
