/***************************************************************************
                               st4jevent.cpp
                             -----------------
    begin                : 04.03.2003
    copyright            : (C) 2003 by Tamas Bara
    email                : tbara@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "st4jevent.h"
#include "transceiver.h"

int SysTray4JavaEvent::SYSTRAY4J_EVENT = 1001;

SysTray4JavaEvent::SysTray4JavaEvent( int toDo,
                                      int menuId,
                                      QString& iconFileName,
                                      QString& toolTip )

    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = new QString( iconFileName );
    m_pToolTip               = new QString( toolTip );
    m_itemIndex              = -1;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId )

    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = -1;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo,
                                      int menuId,
                                      int itemIndex,
                                      QString& itemLabel,
                                      bool checkable,
                                      bool checked,
                                      bool enabled )

    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = itemIndex;
    m_pItemLabel             = new QString( itemLabel );
    m_checkable              = checkable;
    m_check                  = checked;
    m_enable                 = enabled;
    m_showIcon               = false;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId, int itemIndex, QString& itemLabel )

    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = itemIndex;
    m_pItemLabel             = new QString( itemLabel );
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId, QString& toolTipOrIcon )
    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = -1;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;

    if( toDo == Transceiver::SET_TOOLTIP ) m_pToolTip = new QString( toolTipOrIcon );
    else m_pIconFileName = new QString( toolTipOrIcon );
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId, int itemIndex, bool checkOrEnable )
    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = itemIndex;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;

    if( toDo == Transceiver::ENABLE_ITEM ) m_enable = checkOrEnable;
    else m_check = checkOrEnable;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId, int itemIndex )
    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = itemIndex;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = false;
}

SysTray4JavaEvent::SysTray4JavaEvent( int toDo, int menuId, bool showIcon )
    : QCustomEvent( SysTray4JavaEvent::SYSTRAY4J_EVENT )
{
    m_toDo                   = toDo;
    m_menuId                 = menuId;
    m_pIconFileName          = 0;
    m_pToolTip               = 0;
    m_itemIndex              = false;
    m_pItemLabel             = 0;
    m_checkable              = false;
    m_check                  = false;
    m_enable                 = false;
    m_showIcon               = showIcon;
}

SysTray4JavaEvent::~SysTray4JavaEvent()
{
    if( m_pItemLabel ) delete m_pItemLabel;
    if( m_pIconFileName ) delete m_pIconFileName;
    if( m_pToolTip ) delete m_pToolTip;
}
